﻿using GrapeCity.Forguncy.SecurityProvider;
using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Security.Cryptography;
using openWeaverSecurityProvider;
using System.Collections;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;
using System.Data;
using System.Reflection;
using System.IO;
using static WeaverPasswordSecurityProvider.DataSchema;

namespace WeaverPasswordSecurityProvider
{
    public class WeaverPasswordSecurityProvider : ISecurityProvider,ISupportSettings
    {
        public string Name
        {
            get
            {
                return "WeaverPasswordSecurityProvider";
            }
        }
        public AuthenticationType AuthenticationType
        {
            get
            {
                return AuthenticationType.UserNameAndPassword;
            }
        }
        public UserInformationStorageMode UserInformationStorageMode
        {
            get
            {
                return UserInformationStorageMode.InMemoryCache;
            }
        }
        public UserInformations UserInformations
        {
            get
            {
                //创建UserInformations对象
                var userInfo = new UserInformations();
                var config = LoadConfig();
                if (string.IsNullOrEmpty(config.ServerName) || string.IsNullOrEmpty(config.Database) || string.IsNullOrEmpty(config.UserName) || string.IsNullOrEmpty(config.Password))
                {
                    return userInfo;
                }
                //数据库连接字符串
                string constr = "Server=" + config.ServerName + ";database=" + config.Database + ";uid=" + config.UserName + ";pwd=" + config.Password + ";";
                //string constr = "Server=10.32.0.77;database=ecology_demo;uid=sa;pwd=xA123456";
                SqlConnection con = new SqlConnection(constr);
                //打开数据库链接
                con.Open();
                //从数据库中获取用户信息，返回userInfo
                var userTable = GetUser(con, userInfo);
                //从数据库中获取角色信息，返回userInfo
                GetRoles(con, userInfo, userTable);
                //从数据库中获取组织信息，返回userInfo
                GetDepartment(con, userInfo, userTable);
                //关闭数据库连接
                con.Close();
                //将获取到的userInfo返回
                return userInfo;
            }
        }
        private Dictionary<int, User> GetUser(SqlConnection con, UserInformations userInfo)
        {
            string sql = string.Format("select * from HrmResource");
            var dataTable = GetDataTable(sql, con);

            try
            {
                Dictionary<int, User> userDic = new Dictionary<int, User>();

                for (int i = 0; i < dataTable.Rows.Count; i++)
                {
                    var row = dataTable.Rows[i];
                    var user = new User()
                    {
                        UserId = row["loginid"].ToString(),
                        FullName = row["lastname"].ToString(),
                        Email = row["email"].ToString(),
                    };
                    user.Properties.Add("性别", row["sex"].ToString());
                    user.Properties.Add("生日", row["birthday"].ToString());
                    user.Properties.Add("国籍", row["nationality"].ToString());
                    user.Properties.Add("系统语言", row["systemlanguage"].ToString());
                    user.Properties.Add("手机", row["telephone"].ToString());
                    user.Properties.Add("电话", row["mobile"].ToString());

                    userInfo.Users.Add(user);

                    userDic.Add((int)row["id"], user);
                }

                return userDic;
            }
            catch (Exception e)
            {
                throw;
            }
            finally
            {
            }
        }
        private void GetRoles(SqlConnection con, UserInformations userInfo, Dictionary<int, User> userDic)
        {
            var roleTable = GetDataTable("select id, rolesmark from HrmRoles", con);
            var roleMemberTable = GetDataTable("select roleid, resourceid from HrmRoleMembers", con);

            var roleMemberList = GetList(roleMemberTable);

            try
            {
                for (int i = 0; i < roleTable.Rows.Count; i++)
                {
                    var roleRow = roleTable.Rows[i];

                    var role = new Role()
                    {
                        Name = roleRow["rolesmark"].ToString()
                    };

                    var members = roleMemberList.Where(r => object.Equals(r["roleid"], roleRow["id"]));

                    foreach (var item in members)
                    {
                        var uId = (int)item["resourceid"];

                        if (userDic.ContainsKey(uId))
                        {
                            role.Users.Add(userDic[uId]);
                        }
                    }

                    userInfo.Roles.Add(role);
                }
            }
            catch (Exception e)
            {
                throw;
            }
            finally
            {
            }
        }
        private void GetDepartment(SqlConnection con, UserInformations userInfo, Dictionary<int, User> userDic)
        {
            var departmentTable = GetDataTable("select id, departmentmark, supdepid from HrmDepartment", con);
            var userTable = GetDataTable("select id, loginid, departmentid from HrmResource", con);

            var DepartMemberList = GetList(userTable);

            List<OrganizationInfo> orgList = new List<OrganizationInfo>();
            try
            {
                for (int i = 0; i < departmentTable.Rows.Count; i++)
                {
                    var departmentRow = departmentTable.Rows[i];
                    int? superID = null ;
                    try
                    {
                        superID = Convert.ToInt32(departmentRow["supdepid"]);
                    }
                    catch (Exception)
                    {
                        superID = null;
                    }
                    var current = new OrganizationInfo()
                    {
                        ID = Convert.ToInt32(departmentRow["id"]),
                        parentID = superID,
                        Node = new Organization()
                        {
                            Name = departmentRow["departmentmark"].ToString()
                        }
                    };
                    orgList.Add(current);
                }
                userInfo.Organizations.AddRange(AddSubOrganization(null, orgList));
            }
            catch (Exception e)
            {
                throw;
            }
            finally
            {
            }
        }
        private List<Organization> AddSubOrganization(int? parentId, List<OrganizationInfo> allNodes)
        {
            var result = new List<Organization>();

            foreach (var node in allNodes)
            {
                if(node.parentID == parentId)
                {
                    result.Add(node.Node);
                    node.Node.SubOrganizations.AddRange(AddSubOrganization(node.ID, allNodes));
                }
            }

            return result;
        }
        private List<DataRow> GetList(DataTable dataTable)
        {
            List<DataRow> list = new List<DataRow>();
            for (int i = 0; i < dataTable.Rows.Count; i++)
            {
                list.Add(dataTable.Rows[i]);
            }
            return list;
        }
        private DataTable GetDataTable(string sql, SqlConnection con)
        {
            using (SqlCommand com = new SqlCommand(sql, con))
            {
                using (var dataAdapter = new SqlDataAdapter(com))
                {
                    var dataTable = new DataTable();
                    dataAdapter.Fill(dataTable);
                    return dataTable;
                }
            };
        }

        public bool AllowLogout
        {
            get
            {
                return true;
            }
        }

        //public List<SecurityProviderSettings> Settings => new List<SecurityProviderSettings>();
        public List<SecurityProviderSettings> Settings
        {
            get
            {
                var config = LoadConfig();
                var settings = new List<SecurityProviderSettings>();

                settings.Add(new SecurityProviderSettings()
                {
                    Name = "ServerName",
                    Value = config.ServerName
                });
                settings.Add(new SecurityProviderSettings()
                {
                    Name = "Database",
                    Value = config.Database
                });
                settings.Add(new SecurityProviderSettings()
                {
                    Name = "UserName",
                    Value = config.UserName
                });
                settings.Add(new SecurityProviderSettings()
                {
                    Name = "Password",
                    Value = config.Password
                });

                return settings;
            }
        }

        private static Config LoadConfig()
        {
            var assemblyLocation = Assembly.GetExecutingAssembly().Location;
            if (AppDomain.CurrentDomain.ShadowCopyFiles)
            {
                Uri uri = new Uri(Assembly.GetExecutingAssembly().CodeBase);
                assemblyLocation = uri.LocalPath;
            }
            var dir = Path.GetDirectoryName(assemblyLocation);
            var configFile = Path.Combine(dir, "config.json");
            return JsonConvert.DeserializeObject<Config>(File.ReadAllText(configFile));
        }
        public User VerifyUser(Dictionary<string, string> properties)
        {
            var config = LoadConfig();
            var userName = properties["userName"];
            var password = properties["password"];
            string constr = "Server=" + config.ServerName + ";database=" + config.Database + ";uid=" + config.UserName + ";pwd=" + config.Password + ";";
            //string constr = "Server=10.32.0.77;database=ecology_demo;uid=sa;pwd=xA123456";
            SqlConnection con = new SqlConnection(constr);
            con.Open();
            string sql = string.Format("select * from HrmResource where HrmResource.loginid = '{0}'", userName);
            var loginTable = GetDataTable(sql, con);
            try
            {
                for (int i = 0; i < loginTable.Rows.Count; i++)
                {
                    var login = loginTable.Rows[i];
                    var loginid = login["loginid"].ToString();
                    var userpassword = login["password"].ToString();
                    if (userName != null && password != null)
                    {
                        var md5password = GetMD5(password);
                        if (userpassword.ToUpper() == md5password.ToUpper())
                        {
                            return new User
                            {
                                UserId = userName,
                                FullName = userName
                            };
                        }
                    }
                    return null;
                }
                return null;
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                con.Close();
            }
        }
        public void UpdateSetting(Dictionary<string, object> dictionary)
        {
            var config = LoadConfig();

            if (dictionary.ContainsKey("ServerName"))
            {
                var value = dictionary["ServerName"]?.ToString();
                config.ServerName = value;
            }
            if (dictionary.ContainsKey("Database"))
            {
                var value = dictionary["Database"]?.ToString();
                config.Database = value;
            }
            if (dictionary.ContainsKey("UserName"))
            {
                var value = dictionary["UserName"]?.ToString();
                config.UserName = value;
            }
            if (dictionary.ContainsKey("Password"))
            {
                var value = dictionary["Password"]?.ToString();
                config.Password = value;
            }
            SaveConfig(config);
        }
        private void SaveConfig(Config config)
        {
            var assemblyLocation = Assembly.GetExecutingAssembly().Location;
            if (AppDomain.CurrentDomain.ShadowCopyFiles)
            {
                Uri uri = new Uri(Assembly.GetExecutingAssembly().CodeBase);
                assemblyLocation = uri.LocalPath;
            }
            var dir = Path.GetDirectoryName(assemblyLocation);
            var configFile = Path.Combine(dir, "config.json");
            MakeFileNotReadOnly(configFile);
            File.WriteAllText(configFile, JsonConvert.SerializeObject(config));
        }
        public static void MakeFileNotReadOnly(string fileName)
        {
            if (File.Exists(fileName))
            {
                if ((File.GetAttributes(fileName) & FileAttributes.ReadOnly) == FileAttributes.ReadOnly)
                {
                    File.SetAttributes(fileName, File.GetAttributes(fileName) & ~FileAttributes.ReadOnly);
                }
            }
        }
        public static string GetMD5(string str)
        {
            var md5 = System.Security.Cryptography.MD5.Create();
            byte[] data = md5.ComputeHash(Encoding.UTF8.GetBytes(str));

            StringBuilder sBuilder = new StringBuilder();

            for (int i = 0; i < data.Length; i++)
            {
                sBuilder.Append(data[i].ToString("x2"));
            }
            string s = sBuilder.ToString();
            return s;
        }
    }
}